import React from 'react';
import DataListAction from './DataListAction';
import {getDefaultSettings} from '../index';

const LOCALE = {
    zh: {
        actionMenuLabel: '操作',
    },
    en: {
        columnActionTitle: 'Action',
    },
};

/**
 * DataList.ActionMenu component.
 * @memberof DataList
 * @name DataList.ActionMenu
 * @description Define the DataList action menu.
 * @since 0.1.5
 * @author liujiali
 * @property {string} [label=Action] - Action menu label.
 * @property {string} [type=default] - Action menu button type.<br/>[`primary`, `default`]
 * @property {DataList.Action} children - Action of DataList.ActionMenu
 * @example
 * <DataList>
 *     <DataList.Head>
 *         <DataList.ActionMenu>
 *             <DataList.Action label='Add' onClick={(keys) => alert(keys)} />
 *             ......
 *         </DataList.ActionMenu>
 *     </DataList.Head>
 *     ......
 * </DataList>
 */
class DataListActionMenu {
    /**
     * Create and initialize component.
     * @param {*} props - Component props.
     */
    constructor(props) {
        const settings = getDefaultSettings();
        const {locale} = settings;
        const {label, type = 'default', float = 'left'} = props;
        const defaultAttrs = {
            label: label || LOCALE[locale].actionMenuLabel,
            actions: [],
            type: type,
            float: float,
        };
        const {children} = props;
        const actions = [];
        React.Children.map(children, (child) => {
            const ChildComponent = child.type;
            const childIns = new ChildComponent(child.props);
            if (childIns instanceof DataListAction) {
                actions.push(childIns.attrs);
            }
        });
        this.attrs = Object.assign(defaultAttrs, {actions});
    }
}

export default DataListActionMenu;
