import React from 'react';
import {
    isBlank,
    formatNumber,
    formatString,
    convertToLocalDate,
} from 'rc-js-utils';
import {isString, find, isObject, isArray} from 'lodash';
import {Badge} from 'antd';
import {exec} from 'rc-js-utils';
import {Input, InputNumber, Select} from '../Input';

/**
 * DataList.Column component.
 * @memberof DataList
 * @name DataList.Column
 * @description Define the DataList column.
 * @since 0.1.0
 * @author liujiali
 * @property {string} [label=null] - Column label.
 * @property {string} dataKey - Column data key.
 * @property {boolean} [sort=false] - Enable column sort.
 * @property {enum} [defaultSort=null] - Default sort order.
 * @property {render} [render=function(value) { return value ? value : '-' }] - Column render method.
 * @property {boolean} [visible=true] - Whether column is visible.
 * @property {string} [type] - Column type.Effective when type=`list`.
 * @property {string|object} [convert=null] - Several predefined render method.[`date`, `status`, `number`, `string`].
 *                                            <br/>If the `render` property is defined, this props will not be used.
 * @property {enum} convert.type - Convert type.[`date`, `status`, `number`, `string`].
 * @property {string} [convert.format=`YYYY-MM-DD HH:mm:ss`] - Effective when type=`date`.<br/>Date format.
 * @property {number} [convert.timezone=local timezone] - Effective when type=`date`.<br/>Date convert target timezone.
 * @property {object} [convert.statusMap] - Effective when type=`status`.<br/>Status map.
 * @property {string} convert.statusMap.key - Status map key.
 * @property {string} convert.statusMap.value - Status map value.<br/>[`success`, `warning`, `error`, `processing`, `default`].
 * @property {function} [convert.render] - Status label render.
 * @property {string} [help] - Column help message.
 * @example
 * <DataList>
 *     <DataList.Column label='Name' dataKey='name' sort={true} defaultSort='descend' />
 *     <DataList.Column label='Display Name' dataKey='displayName' visible={false} />
 *     <DataList.Column label='Created Time' dataKey='createdTime' convert='date' help='This is when application created.' />
 *     ......
 * </DataList>
 */
class DataListColumn {
    /**
     * Create and initialize component.
     * @param {*} props - Component props.
     */
    constructor(props) {
        const defaultAttrs = {
            label: '',
            dataKey: null,
            sort: false,
            defaultSort: null,
            render: this.getDefaultRender(props),
            visible: true,
            type: null,
            convert: null,
            help: null,
        };
        this.attrs = Object.assign(defaultAttrs, props);
        const {convert, label} = this.attrs;
        if (isObject(convert) && convert.required) {
            this.attrs.label = (<label className='ant-form-item-required'>{label}</label>);
        }
    }
    /**
     * Return default render method for column.
     * @since 0.1.1
     * @author liujiali
     * @private
     * @param {*} props - Component props.
     * @return {render} Column render method.
     */
    getDefaultRender(props) {
        const defaultRender = (value) => isBlank(value) ? '-' : value;
        if (!props.convert) return defaultRender;
        let convert = {};
        isString(props.convert) ?
            convert.type = props.convert : convert = props.convert;
        return this.getConvertRender(convert);
    }
    /**
     * Return render method by convert type.
     * @since 0.1.1
     * @author liujiali
     * @private
     * @param {object} convert - Column convert property.
     * @return {render} Column render method.
     */
    getConvertRender(convert) {
        const type = convert.type;
        let render;
        switch (type) {
        // `string` type.
        case 'string':
            render = (value) => isBlank(value) ? '-' : formatString(value);
            break;
            // `number` type.
        case 'number':
            render = (value) => isBlank(value) ? '-' : formatNumber(value);
            break;
            // `date` type.
        case 'date':
            const {format, timezone} = convert;
            const opts = {};
            if (format) opts.format = format;
            if (!isBlank(timezone)) opts.timezone = timezone;
            render = (value) => isBlank(value) ?
                '-' : convertToLocalDate(value, opts);
            break;
            // `string` type.
        case 'status':
            let {statusMap} = convert;
            if (!statusMap) {
                statusMap = {
                    success: 'success',
                    error: 'error',
                    default: 'default',
                    processing: 'processing',
                    warning: 'warning',
                };
            }
            const statusRender = convert.render || ((val) => formatString(val));
            render = (value) => isBlank(value) ?
                '-' : <Badge status={statusMap[value]}
                    text={exec(statusRender, value)} />;
            break;
        case 'input':
            render = (value, record, index, ins) => {
                const onChange = (val, error) => {
                    record[this.attrs.dataKey] = val;
                    if (!record.fieldsError) {
                        record.fieldsError = {};
                    }
                    record.fieldsError[this.attrs.dataKey] = error;
                    ins.setState({
                        ...ins.state,
                    });
                };
                return <Input initValue={value} trigger={ins.fieldsTrigger}
                    {...convert} onChange={onChange} key={ins.state.loadCount} />;
            };
            break;
        case 'select':
            render = (value, record, index, ins) => {
                const onChange = (value, error) => {
                    record[this.attrs.dataKey] = value;
                    if (!record.fieldsError) {
                        record.fieldsError = {};
                    }
                    record.fieldsError[this.attrs.dataKey] = error;
                    ins.setState({
                        ...ins.state,
                    });
                };
                let {options} = convert;
                options = exec(options, value, record, index);
                let currentOption = undefined;
                if (isArray(value)) {
                    currentOption = [];
                    value.forEach((val) => {
                        if (options.map((i) => i.value).indexOf(val) !== -1) {
                            currentOption.push(val);
                        }
                    });
                    currentOption = currentOption.length ? currentOption : undefined;
                } else {
                    currentOption = find(options, (i) => i.value === value);
                }
                if (value && !currentOption) {
                    record[this.attrs.dataKey] = undefined;
                }
                return <Select initValue={value} trigger={ins.fieldsTrigger}
                    {...convert} onChange={onChange} options={options} key={ins.state.loadCount} />;
            };
            break;
        case 'inputNumber':
            render = (value, record, index, ins) => {
                const onChange = (val, error) => {
                    record[this.attrs.dataKey] = val;
                    if (!record.fieldsError) {
                        record.fieldsError = {};
                    }
                    record.fieldsError[this.attrs.dataKey] = error;
                    ins.setState({
                        ...ins.state,
                    });
                };
                return <InputNumber initValue={value} trigger={ins.fieldsTrigger}
                    {...convert} onChange={onChange} key={ins.state.loadCount} />;
            };
            break;
        default:
            render = (value) => isBlank(value) ? '-' : value;
        };
        return render;
    }
}

export default DataListColumn;

/**
 * render method description.
 * @callback render
 * @param {*} value - The column value.
 * @param {object} record - The whole record value.
 * @param {number} index - The record index.
 */
