import React, {Component, Fragment} from 'react';
import {Select, DatePicker, Input, Button, Row, Col, Tag} from 'antd';
import {find} from 'lodash';
import {exec, isBlank} from 'rc-js-utils';
import moment from 'moment';
import {getDefaultSettings} from '../index';
import {Divider} from 'antd';
import {Collapse} from 'antd';

const {CheckableTag} = Tag;

const LOCALE = {
    zh: {
        filter: '过滤',
        reset: '重置',
        all: '全部',
        filterText: '过滤条件',
    },
    en: {
        filter: 'Filter',
        reset: 'Reset',
        all: 'All',
        filterText: 'Filter condition',
    },
};

/**
 * DataList.Title component.
 * @memberof DataList
 * @name DataList.Filter
 * @description Define the DataList filter.
 * @since 0.1.2
 * @author liujiali
 * @property {string} label - Filter label.
 * @property {string} dataKey - Filter data key.
 * @property {enum} [type=text] - Filter type.<br/>[`text`, `enum`, `date`, `datetime`].
 * @property {function} render - Filter value label render function.
 * @property {Array} [options] - Effextive when type=`enum`.Filter options.
 * @property {string} options.label - Option label.
 * @property {*} options.value - Option value.
 * @example
 * <DataList>
 *     <DataList.Head>
 *         <DataList.Filter>
 *             <DataList.FilterItem label='Name' dataKey='name' type='text' />
 *         </DataList.Filter>
 *     </DataList.Head>
 *     ......
 * </DataList>
 */
class DataListFilter {
    /**
     * Create and initialize component.
     * @param {*} props - Component props.
     */
    constructor(props) {
        const defaultAttrs = {
            children: [],
        };
        const {children, float = 'left', multiple} = props;
        const insProps = {
            children: [],
            float: float,
            multiple: multiple,
        };
        React.Children.map(children, (child) => {
            const {type} = child;
            const Component = type;
            const ChildIns = new Component(child.props);
            insProps.children.push(ChildIns.attrs);
        });
        this.attrs = Object.assign(defaultAttrs, insProps);
    }
}

/**
 * DataList.Title component.
 * @memberof DataList
 * @name DataList.FilterItem
 * @description Define the DataList filter.
 * @since 0.1.2
 * @author liujiali
 * @property {string} label - Filter label.
 * @property {string} dataKey - Filter data key.
 * @property {enum} [type=text] - Filter type.<br/>[`text`, `enum`, `date`, `datetime`].
 * @property {function} render - Filter value label render function.
 * @property {Array} [options] - Effextive when type=`enum`.Filter options.
 * @property {string} options.label - Option label.
 * @property {*} options.value - Option value.
 * @example
 * <DataList>
 *     <DataList.Head>
 *         <DataList.Filter>
 *             <DataList.FilterItem label='Name' dataKey='name' type='text' />
 *             ......
 *         </DataList.Filter>
 *     </DataList.Head>
 *     ......
 * </DataList>
 */
class DataListFilterItem {
    /**
     * Create and initialize component.
     * @param {*} props - Component props.
     */
    constructor(props) {
        const defaultAttrs = {
            label: '',
            dataKey: null,
            type: 'text',
        };
        this.attrs = Object.assign(defaultAttrs, props);
    }
}

/**
 * DataList filter node.
 * @private
 */
class DataListFilterComponent extends Component {
    /**
     * Create and initialize component.
     * @param {*} props - Component props.
     */
    constructor(props) {
        super(props);
        const {filter, multiple} = props;
        if (multiple) {
            this.state = {};
            filter.forEach((f) => {
                const {label, dataKey, type, render, options} = f;
                this.state[dataKey] = {   // eslint-disable-line
                    label: label,
                    dataKey: dataKey,
                    valueLabel: null,
                    value: null,
                    type: type,
                    render: render || this.getFilterRender(type, options || []),
                    options: options || [],
                };
            });
        } else {
            const defaultFilter = filter[0];
            const {label, dataKey, type, render, options} = defaultFilter;
            this.state = {
                label: label,
                dataKey: dataKey,
                valueLabel: null,
                value: null,
                type: type,
                render: render || this.getFilterRender(type, options || []),
                options: options || [],
            };
        }
    }
    /**
     * Get default filter value label render.
     * @since 0.1.4
     * @author liujiali
     * @private
     * @param {string} type - Filter type.
     * @param {Array} options - Filter options.Effective when type=`enum`.
     * @return {function} Filter value label render.
     */
    getFilterRender(type, options) {
        let render = null;
        switch (type) {
        case 'date':
            render = (dates) => {
                if (!dates.length) return null;
                const start = dates[0].format('YYYY-MM-DD');
                const end = dates[1].format('YYYY-MM-DD');
                return `${start} ~ ${end}`;
            };
            break;
        case 'datetime':
            render = (dates) => {
                if (!dates.length) return null;
                const start = dates[0].format('YYYY-MM-DD HH:mm:ss');
                const end = dates[1].format('YYYY-MM-DD HH:mm:ss');
                return `${start} ~ ${end}`;
            };
            break;
        case 'enum':
            render = (value) => {
                const result = find(options, (i) => i.value === value);
                return result.label;
            };
            break;
        default:
            render = (val) => val;
        }
        return render;
    }
    /**
     * Handle filter change.
     * @since 0.1.4
     * @author liujiali
     * @private
     * @return {function} filter select change handler.
     */
    handleSelectChange() {
        return (dataKey) => {
            const {filter} = this.props;
            const current = find(filter, (i) => i.dataKey === dataKey);
            this.setState({
                label: current.label,
                dataKey: current.dataKey,
                valueLabel: null,
                value: null,
                type: current.type || 'text',
                render: current.render || this.getFilterRender(
                    current.type || 'text', current.options || []
                ),
                options: current.options || [],
            });
        };
    }
    /**
     * Get filter component by type.
     * @since 0.2.6
     * @author liujiali
     * @private
     * @param  {enum} filterType - Component type.
     * @param  {any} model - Filter component model.
     * @return {ReactNode} - Filter component.
     */
    getFilterComponent(filterType, model) {
        let filterInput = null;
        const {value, options = [], dataKey} = model;
        const settings = getDefaultSettings();
        const {locale} = settings;
        switch (filterType) {
        case 'date':
            filterInput = (
                <DatePicker.RangePicker value={value}
                    onChange={(dates) => {
                        this.handleFilterChange(dates, dataKey);
                    }}
                    style={{width: 240}} />
            );
            break;
        case 'datetime':
            const defaultValue = [
                moment('00:00:00', 'HH:mm:ss'),
                moment('23:59:59', 'HH:mm:ss'),
            ];
            filterInput = (
                <DatePicker.RangePicker value={value}
                    onChange={(dates) => {
                        this.handleFilterChange(dates, dataKey);
                    }}
                    style={{width: 320}}
                    format='YYYY-MM-DD HH:mm:ss'
                    showTime={{defaultValue, format: 'HH:mm:ss'}} />
            );
            break;
        case 'enum':
            filterInput = (
                <Select value={value}
                    onChange={(value) => {
                        this.handleFilterChange(value, dataKey);
                    }}
                    style={{minWidth: 200}}>
                    {options.map((opt) => {
                        return (
                            <Select.Option value={opt.value}
                                key={opt.value}>{opt.label}</Select.Option>
                        );
                    })}
                </Select>
            );
            break;
        case 'multiple':
            const onTagsSelect = (val, checked) => {
                const currentValue = [].concat(value || []);
                if (!checked) {
                    const index = value.indexOf(val);
                    currentValue.splice(index, 1);
                } else {
                    currentValue.push(val);
                }
                this.handleFilterChange(
                    currentValue.length ? currentValue : null, dataKey
                );
            };
            const onSelectAll = (checked) => {
                let currentValue = [];
                if (checked) {
                    currentValue = options.map((opt) => opt.value);
                }
                this.handleFilterChange(checked ? currentValue : null, dataKey);
            };
            const checkedList = value || [];
            filterInput = (
                <Fragment>
                    <CheckableTag
                        checked={checkedList.length === options.length}
                        onChange={(checked) => onSelectAll(checked)}>
                        {LOCALE[locale].all}
                    </CheckableTag>
                    {options.map((opt, index) => {
                        return (
                            <CheckableTag
                                key={index}
                                checked={checkedList.indexOf(opt.value) > -1}
                                onChange={(checked) => {
                                    onTagsSelect(opt.value, checked);
                                }}>{opt.label}
                            </CheckableTag>
                        );
                    })}
                </Fragment>
            );
            break;
        default:
            filterInput = (
                <Input value={value}
                    style={{width: 200}}
                    onChange={(e) => {
                        this.handleFilterChange(e.target.value, dataKey);
                    }} />
            );
        }
        return filterInput;
    }
    /**
     * Get filter input component.
     * @since 0.1.4
     * @author liujiali
     * @private
     * @param {string} filterType - Filter type.
     * @return {ReactNode} Filter input node.
     */
    renderFilterInput(filterType) {
        return (
            <Input.Group compact style={{width: 'auto'}}>
                {this.getFilterComponent(filterType, this.state)}
                <Button type='primary'
                    icon='search'
                    onClick={this.handleSearch()} />
            </Input.Group>
        );
    }
    /**
     * Handle filter input change.
     * @since 0.1.4
     * @author liujiali
     * @private
     * @param {*} value - Filter value.
     * @param {string} dataKey - Component data key.
     */
    handleFilterChange(value, dataKey) {
        const {multiple} = this.props;
        if (multiple) {
            const targetState = this.state[dataKey];
            const {render} = targetState;
            this.setState({
                [dataKey]: {
                    ...targetState,
                    value: value,
                    valueLabel: exec(render, value),
                },
            });
        } else {
            this.setState({
                value: value,
                valueLabel: exec(this.state.render, value),
            });
        }
    }
    /**
     * Handle filter button click.
     * @since 0.1.4
     * @author liujiali
     * @private
     * @return {function} Filter button click handler.
     */
    handleSearch() {
        return () => {
            const emptyVal = isBlank(this.state.value);
            // type=`date` or type=`datetime`
            const emptyArray = this.state.value.length === 0;
            if (emptyVal || emptyArray) return;
            const {onSearch} = this.props;
            exec(onSearch, this.state);
            this.setState({
                value: null,
                valueLabel: null,
            });
        };
    }
    /**
     * Handle filter button click.
     * @since 0.2.6
     * @author liujiali
     * @private
     * @return {function} Filter button click handler.
     */
    handleMultipleSearch() {
        return (e) => {
            e.stopPropagation();
            const {onMultipleSearch} = this.props;
            const filters = [];
            for (const key in this.state) {
                if (!isBlank(this.state[key].value)) {
                    filters.push(this.state[key]);
                }
            }
            exec(onMultipleSearch, filters);
        };
    }
    /**
     * Handle reset button click.
     * @since 0.2.6
     * @author liujiali
     * @private
     * @return {function} Reset button click handler.
     */
    handleResetFilterFields() {
        return (e) => {
            e.stopPropagation();
            const nextState = {};
            for (const key in this.state) {
                if (key) {
                    nextState[key] = {
                        ...this.state[key],
                        value: null,
                        valueLabel: null,
                    };
                }
            }
            this.setState({
                ...nextState,
            });
        };
    }
    /**
     * Render single filter.
     * @since 0.2.6
     * @author liujiali
     * @private
     * @return {ReactNode} - Single filter component.
     */
    renderSingleFilter() {
        const {filter} = this.props;
        const {dataKey, type} = this.state;
        const filterType = type || filter[0].type;
        const select = (
            <Select defaultValue={dataKey}
                dropdownMatchSelectWidth={false}
                onChange={this.handleSelectChange()}
                style={{marginRight: 5}}>
                {filter.map((ftr) => {
                    return (
                        <Select.Option value={ftr.dataKey} key={ftr.dataKey}>
                            {ftr.label}
                        </Select.Option>
                    );
                })}
            </Select>
        );
        return (
            <Fragment>
                {select}
                {this.renderFilterInput(filterType)}
            </Fragment>
        );
    }
    /**
     * Handle event stopPropagation;
     * @since 0.4.5
     * @author liujiali
     * @private
     * @return {function} - Event handler.
     */
    handleCollaspe() {
        return (e) => {
            if (e.target.nodeName !== 'SPAN') {
                e.stopPropagation();
            }
        };
    }
    /**
     * Render multiple filter.
     * @since 0.2.6
     * @author liujiali
     * @private
     * @return {ReactNode} - Multiple filter component.
     */
    renderMultipleFilter() {
        const {filter} = this.props;
        const colList = [];
        let rowCol = [];
        filter.forEach((f, index) => {
            const {span = 3} = f;
            const rowSpan = rowCol.reduce((total, currentValue) => {
                total += currentValue.span || 3;
                return total;
            }, 0);
            if (rowSpan + span > 3) {
                colList.push([].concat(rowCol));
                rowCol = [];
            }
            rowCol.push(f);
            if (index >= filter.length - 1) {
                colList.push(rowCol);
            }
        });
        const settings = getDefaultSettings();
        const {locale} = settings;
        const lineStyle = {
            padding: '5px 8px',
            borderBottom: '1px solid #fff',
        };
        const colStyle = {
            display: 'flex',
            height: 42,
            alignItems: 'center',
            padding: '0 10px',
        };
        const collapseStyle = {
            margin: '-12px -16px -12px -24px',
            padding: '12px 16px 12px 24px',
            cursor: 'default',
        };
        return (
            <Fragment>
                {/* <Row style={{marginBottom: 10}}>
                    <Button
                        type='primary'
                        onClick={this.handleMultipleSearch()}>
                        {LOCALE[locale].filter}
                    </Button>
                    <Button style={{marginLeft: 5}}
                        onClick={this.handleResetFilterFields()}>
                        {LOCALE[locale].reset}
                    </Button>
                </Row> */}
                <Collapse bordered={false} className='rc-dataList-filter'>
                    <Collapse.Panel header={(
                        <Row type='flex' align='middle'
                            style={collapseStyle} onClick={this.handleCollaspe()}>
                            <span style={{cursor: 'pointer'}}>{LOCALE[locale].filterText}</span>
                            <Divider type='vertical' />
                            <Button
                                icon='filter'
                                // type='primary'
                                onClick={this.handleMultipleSearch()}>
                                {LOCALE[locale].filter}
                            </Button>
                            <Button style={{marginLeft: 10}}
                                icon='rest'
                                onClick={this.handleResetFilterFields()}>
                                {LOCALE[locale].reset}
                            </Button>
                        </Row>
                    )}>
                        <Row style={{background: '#f5f5f5', marginBottom: 10}}>
                            {colList.map((row, index) => {
                                return (
                                    <Row style={lineStyle} key={index}>
                                        {row.map((col, index) => {
                                            const {labelWidth = 80, colSpan = 8} = col;
                                            const width = `calc(100% - ${labelWidth}px)`;
                                            return (
                                                <Col span={colSpan} style={colStyle} key={index}>
                                                    <Col span={1} style={{width: labelWidth}}>
                                                        {col.label}:
                                                    </Col>
                                                    <Col span={1} style={{width}}>
                                                        {this.getFilterComponent(
                                                            col.type, this.state[col.dataKey]
                                                        )}
                                                    </Col>
                                                </Col>
                                            );
                                        })}
                                    </Row>
                                );
                            })}
                        </Row>
                    </Collapse.Panel>
                </Collapse>
            </Fragment>
        );
    }
    /**
     * render method.
     * @return {*} No return value.
     */
    render() {
        const {multiple} = this.props;
        return multiple ?
            this.renderMultipleFilter() : this.renderSingleFilter();
    }
}

export default DataListFilter;
export {DataListFilterItem};
export {DataListFilterComponent};
