import React from "react";
import DataListBasic from "./DataListBasic";
import { Table, Divider, Menu, Tooltip, Icon, Dropdown } from "antd";
import { getStrLength, exec } from "rc-js-utils";
import { isString, isEmpty, clone,isBoolean } from "lodash";
import { getDefaultSettings } from "../index";


const LOCALE = {
  zh: {
    columnActionTitle: "操作",
    columnActionMore: "更多",
    noData: "暂无数据"
  },
  en: {
    columnActionTitle: "Action",
    columnActionMore: "More",
    noData: "No Data"
  }
};

const displayActions = 3;
/**
 * @name DataListTable
 * @description DataList Table.
 * @since 0.1.0
 * @author liujiali
 * @private
 */
class DataListTable extends DataListBasic {
  /**
   * Create and initialize component.
   * @param {*} props - Component props.
   */
  constructor(props) {
    super(props);
    this.state = {};
  }
  /**
   * Convert columns to antd Table columns definition.
   * @since 0.1.0
   * @author liujiali
   * @param {Array} columns - DataList columns definitons.
   * @return {Array} antd Table columns definition.
   */
  convertColumn(columns) {
    return columns.map(col => {
      const render = col.render || col.convert;
      const { instance } = this.props;
      return {
        ...col,
        title: col.label,
        key: col.dataKey,
        dataIndex: col.dataKey,
        render: (val, record, index) =>
          exec(render, val, record, index, instance),
        sorter: col.sort,
        defaultSortOrder: col.defaultSort
      };
    });
  }
  /**
   * Convert actions to antd Table column.
   * @since 0.1.0
   * @author liujiali
   * @param {Array} actions - DataList actions definitons.
   * @return {object} antd Table columns definition.
   */
  convertAction(actions) {
    if (!actions.length) return [];
    const settings = getDefaultSettings();
    const { locale } = settings;
    const actionColumn = {
      title: LOCALE[locale].columnActionTitle,
      dataIndex: "DataList.Action",
      key: "DataList.Action",
      render: this.renderAction(actions),
      width: this.getActionWidth(actions)
    };
    return actionColumn;
  }
  /**
   * Return `render` method for action column.
   * @this DataListTable
   * @since 0.1.0
   * @author liujiali
   * @param {Array} actions - DataList actions.
   * @return {render} antd Table columns render function.
   *                  <br/>For more detail, please see https://ant.design/components/table-cn/#Column
   */
  renderAction(actions) {
    return (value, record, index) => {
      const settings = getDefaultSettings();
      const { locale } = settings;
      const actionMenus = [];
      // Whether to fold part of the actions
      const columnActionCollapse = actions.length > displayActions;
      // Get display actions
      const columnActionList = actions.slice(
        0,
        columnActionCollapse ? displayActions - 1 : displayActions
      );
      columnActionList.forEach((colAction, i) => {
        if (i) {
          actionMenus.push(<Divider type="vertical" key={i + 100} />);
        }
        actionMenus.push(this.renderActionItem(record, index, colAction, i));
      });
      // Set the actions in 'More' menu
      if (columnActionCollapse) {
        const moreMenuList = actions.slice(2);
        const moreLocaleText = LOCALE[locale].columnActionMore;
        const moreMenu = (
          <Menu>
            {moreMenuList.map((menu, i) => (
              <Menu.Item key={i}>
                {this.renderActionItem(record, index, menu, i)}
              </Menu.Item>
            ))}
          </Menu>
        );
        actionMenus.push(<Divider type="vertical" key={200} />);
        // `+ value` is just to prevent eslint error.
        actionMenus.push(
          <Dropdown overlay={moreMenu} key={201}>
            <a>
              {moreLocaleText} <Icon type="down" />
            </a>
          </Dropdown>
        );
      }
      return actionMenus;
    };
  }
  /**
   * Render DataList action.
   * @since 0.1.0
   * @author liujiali
   * @param {object} record - Column record.
   * @param {number} index - Column record index.
   * @param {object} action - Action definition.
   * @param {number} i - Action index.
   * @return {ReactNode} ReactNode for DataList action.
   */
  renderActionItem(record, index, action, i) {
    let { disabled } = action;
    disabled = exec(disabled, record, index);
    const message = disabled.isDisabled ? disabled.message : null;
    const disabledResult = isBoolean(disabled)
    ? disabled
    : disabled && disabled.isDisabled;
    return (
      <Tooltip title={message} key={i}>
        {/* <span>
                    <a disabled={disabled.isDisabled}
                        onClick={() => action.onClick(record, index)}>
                        {action.label}
                    </a>
                </span> */}
        {disabledResult ? (
          <span>
            <a disabled>{action.label}</a>
          </span>
        ) : (
          <span>
            <a
              onClick={() => action.onClick(record, index)}
            >
              {action.label}
            </a>
          </span>
        )}
      </Tooltip>
    );
  }
  /**
   * Calculate actions width by string length.
   * @since 0.1.0
   * @author liujiali
   * @param {Array} actions - DataList actions.
   * @return {number} DataList action labels width.
   */
  getActionWidth(actions) {
    const settings = getDefaultSettings();
    const { locale } = settings;
    let displayActionStr = "";
    const actionCollapse = actions.length > displayActions;
    const actionLabels = actions
      .slice(0, actionCollapse ? displayActions - 1 : displayActions)
      .map(i => i.label);
    // Resolve `More <Icon type='down' />` width
    const moreStr = LOCALE[locale].columnActionMore;
    if (actionCollapse) actionLabels.push(`${moreStr} DD`);
    actionLabels.forEach((act, index) => {
      // Resolve `<Divider type='vertical' />` width
      if (index) {
        displayActionStr += "  ";
      }
      displayActionStr += act;
    });
    // Get action label length
    let actionLen = getStrLength(displayActionStr) * 8;
    // Resolve `td` padding
    actionLen += 16 * 2;
    return actionLen > 120 ? actionLen : 120;
  }
  /**
   * antd Table onChange hook function.
   * @since 0.1.0
   * @author liujiali
   * @return {function} antd Table onChange handle function.
   */
  onChange() {
    const { onChange, pagination, filters } = this.props;
    return (tablePagination, tableFilters, tableSorter) => {
      const DataListPagination = {
        currentPage: tablePagination.current,
        pageSize: tablePagination.pageSize,
        total: pagination.total
      };
      const DataListFilters = filters;
      const DataListSorts = isEmpty(tableSorter)
        ? []
        : [
            {
              key: tableSorter.columnKey,
              value: tableSorter.order
            }
          ];
      exec(onChange, DataListPagination, DataListFilters, DataListSorts);
    };
  }

  /**
   * Render method.
   * @since 0.1.0
   * @return {*} No return value.
   */
  render() {
    const {
      data,
      columns,
      actions,
      loading,
      pagination,
      onSelect,
      selectedKeys
    } = this.props;
    const settings = getDefaultSettings();
    const { locale } = settings;
    const tableColumns = this.convertColumn(columns);
    const columnActions = this.convertAction(actions);
    const showTableColumns = tableColumns.concat(columnActions);
    let { rowSelection } = this.props;
    rowSelection = isString(rowSelection)
      ? { type: rowSelection }
      : rowSelection;
    const tableRowSelection = clone(rowSelection);
    if (rowSelection) {
      tableRowSelection.onChange = onSelect;
      tableRowSelection.selectedRowKeys = selectedKeys;
    }
    return (
      <Table
        bordered={true}
        {...this.props}
        columns={showTableColumns}
        dataSource={data}
        loading={loading}
        locale={{ emptyText: LOCALE[locale].noData }}
        pagination={pagination}
        rowSelection={tableRowSelection}
        onChange={this.onChange()}
      />
    );
  }
}

export default DataListTable;
