import React, {Component, Fragment} from 'react';
import DataListColumn from './DataListColumn';
import DataListAction from './DataListAction';
import DataListPagination from './DataListPagination';
import DataListTable from './DataListTable';
import {isBlank, exec} from 'rc-js-utils';
import {isFunction, isArray, find, isEmpty, isObject, isEqual} from 'lodash';
import DataListTitle from './DataListTitle';
import DataListHead from './DataListHead';
import DataListRefresh from './DataListRefresh';
import {Row, Button, Icon, Tag, Tooltip, Menu, Dropdown} from 'antd';
import DataListFilter, {DataListFilterItem, DataListFilterComponent} from './DataListFilter';
import DataListActionMenu from './DataListActionMenu';
import {Subject} from 'rxjs';

const headMarginMap = {
    left: {marginRight: 8},
    right: {marginLeft: 8},
};

/**
 * DataList component.
 * @namespace DataList
 * @description Component lifecycle:
 * <br/> componentDidMount --> loadTrigger --> beforeLoadData --> handleRequestData --> startLoadingData --> handleResponseData --> afterLoadData --> render
 * @property {enum} [type=`table`] - DataList type.<br/>[`table`, `list`, `card`].
 * @property {*} data - The DataList component `data` used for `handleRequestData` function.
 * @property {beforeLoadData} beforeLoadData - Hook function before start loading data.
 * @property {afterLoadData} afterLoadData - Hook function after loading data.
 * @property {enum|object} [rowSelection] - Properties for row selection.[`checkbox`, `radio`].
 *                                          <br/>For more detail, please see https://ant.design/components/table/#rowSelection.
 * @example
 * <DataList type='table' data={{url: '/api/test'}} rowSelection='checkbox'>
 * ......
 * </DataList>
*/
class DataList extends Component {
    /**
     * Create and initialize component.
     * @param {*} props - Component props.
     */
    constructor(props) {
        super(props);
        let pagination = this.getDefineFromProps(
            props.children, DataListPagination
        );
        const columns = this.getDefineFromProps(
            props.children, DataListColumn
        );
        const sorts = [];
        columns.forEach((col) => {
            if (col.defaultSort) {
                sorts.push({key: col.dataKey, value: col.defaultSort});
            }
        });
        pagination = pagination.length ?
            pagination[0] : false;
        this.fieldsTrigger = new Subject();
        this.state = {
            filters: [],
            sorts: sorts,
            pagination: {
                currentPage: 1,
                pageSize: pagination ? pagination.pageSize : 0,
                total: 0,
            },
            loading: false,
            data: [],
            // TODO: default value
            selectedKeys: [],
            selectedRows: [],
            loadData: () => this.loadData(),
            props,
            columns,
            loadCount: 0,
        };
    };
    /**
     * componentDidMount hook function.
     * @private
     */
    componentDidMount() {
        this.loadTriggerCallback = exec(
            DataList.loadTrigger, this.startLoadingData()
        );
    };
    /**
     * componentWillUnmount hook function.
     * @private
     */
    componentWillUnmount() {
        exec(this.loadTriggerCallback);
    };
    /**
     * componentDidUpdate hook function.
     * @param {any} prevProps - Previous props.
     * @private
     */
    componentDidUpdate(prevProps) {
        if (!isEqual(prevProps.data, this.props.data)) {
            this.loadData(true);
        }
    }
    static Head = DataListHead;
    static Refresh = DataListRefresh;
    static Filter = DataListFilter;
    static FilterItem = DataListFilterItem;
    static Title = DataListTitle;
    static Column = DataListColumn;
    static Action = DataListAction;
    static Pagination = DataListPagination;
    static ActionMenu = DataListActionMenu;
    /**
     * DataList triggers loadData method.This method will be executed when componentDidMount.
     * @since 0.1.0
     * @author liujiali
     * @param {function} loadData - DataList loadData function.
     * @return {function} Stop triggers loadData.The function will be executed when componentWillUnmount.
     * @example
     *
     * DataList.loadTrigger = (loadData) => {
     *     const intervalNo = setInterval(() => loadData(true), 3000);
     *     return () => clearInterval(intervalNo);
     * }
     */
    static loadTrigger(loadData) {
        loadData();
        return null;
    }
    /**
     * @todo  Support observable in future.
     * @description This method is used to handle the datasource.
     *              <br/>For example, if the data comes from the Server,
     *              return the response data or just the request promise like `return fetch(data.url)`.
     *              <br/>If you are looking for the callback function of the request promise, please see `handleResponseData` method.
     * @since 0.1.0
     * @author liujiali
     * @param {*} data - props `data` in DataList.
     * @param {object} pagination - Pagination information.
     * @param {number} pagination.currentPage - Current page number.<br/>Start from 1.
     * @param {number} pagination.pageSize - Number of data items per page.
     * @param {number} pagination.total - Total item count.
     * @param {object[]} filters - Current table filter items.
     * @param {string} filters.key - Filter column key.
     * @param {string} filters.label - Filter column label.
     * @param {string} filters.value - Filter column value.
     * @param {string} filters.valueLabel - Filter column value label.
     * @param {enum} filters.type - Filter type.<br/>[`text`, `range`, `enum`, `value`].
     * @param {object[]} sorts - Current table sort items.
     * @param {string} sorts.key - Sort column key.
     * @param {enum} sorts.value - Sort column order.<br/>[`ascend`, `descend`].
     * @param {...args} args - Extra parameters.
     * @return {object} Return data for DataList component. Support object and promise.
     * @example
     * DataList.handleRequestData = (data, pagination, filters, sorts) => {
     *     return getListData(data, {
     *         currentPage: pagination.currentPage,
     *         pageSize: pagination.pageSize,
     *         sort: sorts.map(i => {
     *             i.value = i.value.replace(/end/g, '');
     *             return i;
     *         }),
     *         filter: filters
     *     })
     * }
     */
    static handleRequestData(data, pagination, filters, sorts, ...args) {
        return {
            pagination: pagination,
            filters: filters,
            sorts: sorts,
            data: data,
        };
    };
    /**
     * @description This method is uesd to handle the response data before render.
     * @since 0.1.0
     * @author liujiali
     * @param  {*} response - Resopnse data.
     * @param  {object} pagination - Pagination information.
     * @param  {number} pagination.currentPage - Current page number.<br/>Start from 1.
     * @param  {number} pagination.pageSize - Number of data items per page.
     * @param {number} pagination.total - Total item count.
     * @param  {object[]} filters - Current table filter items.
     * @param  {string} filters.key - Filter column key.
     * @param {string} filters.label - Filter column label.
     * @param  {string} filters.value - Filter column value.
     * @param {string} filters.valueLabel - Filter column value label.
     * @param  {enum} filters.type - Filter type.<br/>[`text`, `range`, `enum`, `value`].
     * @param  {object[]} sorts - Current table sort items.
     * @param  {string} sorts.key - Sort column key.
     * @param  {enum} sorts.value - Sort column order.<br/>[`ascend`, `descend`].
     * @param {*} data - DataList `data` props value.
     * @return  {object} Return data for DataList component.The data should contain data, pagination, filters and sorts key as input parameters describe.
     * @example
     * DataList.handleResponseData = (response, pagination, filters, sorts) => {
     *     pagination.total = response.allCount;
     *     return {
     *         data: response,
     *         pagination,
     *         filters,
     *         sorts
     *     }
     * }
     */
    static handleResponseData(response, pagination, filters, sorts, data) {
        return {
            pagination: pagination,
            filters: filters,
            sorts: sorts,
            data: isArray(response) ? response : [],
        };
    };
    /**
     * Get the definition of column, action and etc. by parameters.
     * @since 0.1.0
     * @author liujiali
     * @private
     * @param  {ReactNode|ReactNode[]} children - Component children nodes.
     * @param  {class} targetClass - Component class.
     * @return {Array} Component definition list.
     */
    getDefineFromProps(children, targetClass) {
        const attrs = [];
        React.Children.map(children, (child) => {
            const ChildComponent = child.type;
            const childIns = new ChildComponent(child.props);
            if (childIns instanceof targetClass) {
                attrs.push(childIns.attrs);
            }
        });
        return attrs;
    };
    /**
     * Load data function.
     * @since 0.1.0
     * @author liujiali
     * @private
     * @return {callback} startLoadingData function.
     */
    startLoadingData() {
        return (silent) => {
            const {data, beforeLoadData, stopSilentLoad} = this.props;
            if (stopSilentLoad && silent) return;
            const {pagination, filters, sorts} = this.state;
            // Set table loading status by `silent`.
            this.setState({
                loading: !silent,
            });
            let requestParams = [data, pagination, filters, sorts];
            if (isFunction(beforeLoadData)) {
                requestParams = beforeLoadData(
                    data, pagination, filters, sorts
                );
            }
            // Start load data.
            const response = DataList.handleRequestData(...requestParams);
            if (response instanceof Promise) {
                response.then((result) => {
                    this.handleLoadData(result, silent);
                });
            } else {
                this.handleLoadData(response, silent);
            }
        };
    }
    /**
     * Completed unset parameter if developers do not return the
     * `pagination`, `filters`, `sorts` parameters in `afterLoadData` and `handleResponseData` function.
     * @since 0.1.0
     * @author liujiali
     * @deprecated
     * @private
     * @param  {*} response - startLoadingData response.
     * @return {object} - DataList expected return value.
     */
    handleUnsetResult(response) {
        let {data, pagination, filters, sorts} = response;
        if (isBlank(data)) data = [];
        if (isBlank(pagination)) pagination = this.state.pagination;
        if (isBlank(filters)) filters = this.state.filters;
        if (isBlank(sorts)) sorts = this.state.sorts;
        return {
            data,
            pagination,
            filters,
            sorts,
        };
    }
    /**
     * Hook function after `handleResponseData` function.
     * @since 0.1.0
     * @author liujiali
     * @private
     * @param {*} response - startLoadingData response.
     * @param {boolean} silent - Show loading or not.
     */
    handleLoadData(response, silent) {
        const {afterLoadData, data} = this.props;
        const {pagination, filters, sorts, loadCount} = this.state;
        response = DataList.handleResponseData(
            response, pagination, filters, sorts, data
        );
        let tData;
        let tPagination;
        let tFilters;
        let tSorts;
        if (isFunction(afterLoadData)) {
            tData = response.data;
            tPagination = response.pagination;
            tFilters = response.filters;
            tSorts = response.sorts;
            response = afterLoadData(
                tData, tPagination, tFilters, tSorts, data
            );
        }
        Promise.all([response]).then((results) => {
            const result = results[0];
            tData = result.data;
            tPagination = result.pagination;
            tFilters = result.filters;
            tSorts = result.sorts;
            const {rowKey} = this.props;
            const {selectedKeys, selectedRows} = this.state;
            let currentSelectedKeys = selectedKeys;
            let currentSelectedRows = selectedRows;
            if (rowKey) {
                const currentRowKeys = tData.map((i) => i[rowKey]);
                currentSelectedKeys = selectedKeys.filter((i) => {
                    currentRowKeys.indexOf(i) !== -1;
                });
                currentSelectedRows = selectedRows.filter((i) => {
                    currentRowKeys.indexOf(i[rowKey]) !== -1;
                });
            }
            this.setState({
                loading: false,
                filters: tFilters,
                sorts: tSorts,
                pagination: tPagination,
                data: tData,
                loadCount: silent ? loadCount : loadCount + 1,
                selectedKeys: currentSelectedKeys,
                selectedRows: currentSelectedRows,
            });
        });
    }
    /**
     * render DataList componet.
     * @since 0.1.0
     * @author liujiali
     * @private
     * @param {boolen} condition - Whether render component.
     * @param {ReactNode} Component - DataList component.
     * @return {ReactNode} DataList component.
     */
    renderComponet(condition, Component) {
        if (condition) {
            const {children} = this.props;
            const columns = this.getDefineFromProps(children, DataListColumn);
            const columnActions = this.getDefineFromProps(
                children, DataListAction
            );
            let pagination = this.getDefineFromProps(
                children, DataListPagination
            );
            pagination = pagination.length ?
                pagination[0] : false;
            // Handle pagination.
            if (pagination) {
                const {currentPage, pageSize, total} = this.state.pagination;
                const {defaultPageSize} = pagination;
                pagination.current = currentPage;
                pagination.pageSize = pageSize || defaultPageSize;
                pagination.total = total;
            }
            return (
                <Component {...this.props}
                    data={this.state.data}
                    columns={columns}
                    actions={columnActions}
                    loading={this.state.loading}
                    pagination={pagination}
                    filters={this.state.filters}
                    sorts={this.state.sorts}
                    instance={this}
                    selectedKeys={this.state.selectedKeys}
                    onChange={this.handleDataListChange()}
                    onSelect={this.handleDataListSelect()} />
            );
        } else {
            return null;
        }
    }
    /**
     * @since 0.1.2
     * @author liujiali
     * @private
     * @param {object} title - DataList.Title props.
     * @return {ReactNode} - DataList.Title ReactNode.
     */
    renderComponentTitle(title) {
        const titleStyle = {
            position: 'relative',
            marginBottom: 16,
            fontSize: 20,
            lineHeight: '28px',
        };
        const titleClassName = 'rc-ui-component-datalist-title';
        return (
            <Fragment>
                {title.title ? <div className={titleClassName}
                    style={titleStyle}>{title.title}</div> : null}
                {title.children ? title.children : null}
            </Fragment>
        );
    }
    /**
     * Render DataList head.
     * @since 0.1.2
     * @author liujiali
     * @private
     * @param  {object} head - DataList.Head props.
     * @return {ReactNode} DataList head ReactNode.
     */
    renderComponentHead(head) {
        if (isEmpty(head)) return null;
        // Define head in-line style.
        const headStyle = {
            height: 'auto',
            marginBottom: 6,
        };
        const noHead = !head.length;
        const filter = find(head, (h) => h.type === 'filter');
        return noHead ? null : (
            <Fragment>
                <Row style={headStyle}>
                    {head.map((component) => {
                        switch (component.type) {
                        case 'refresh':
                            return this.renderComponentHeadRefresh(component.attrs);
                        case 'filter':
                            return this.renderComponentHeadFilters(component.attrs);
                        case 'action':
                            return this.renderComponentHeadActions(component.attrs);
                        case 'menu':
                            return this.renderComponentHeadMenus(component.attrs);
                        default:
                            return null;
                        }
                    })}
                </Row>
                <Row style={{marginBottom: 6}}>
                    {filter && !filter.attrs.multiple ? this.renderFilterTags() : null}
                </Row>
            </Fragment>
        );
    }
    /**
     * Render DataList.Head refresh button.
     * @since 0.1.2
     * @author liujiali
     * @private
     * @param {object} refresh - DataList.Refresh props.
     * @return {ReactNode} DataList.Refresh node.
     */
    renderComponentHeadRefresh(refresh) {
        if (!refresh) return null;
        const {label, float} = refresh;
        const style = {
            float: float || 'left',
            ...headMarginMap[float || 'left'],
        };
        const refreshNode = (
            <div style={style}>
                <Button icon={label ? null : 'redo'}
                    onClick={() => this.startLoadingData()()}>
                    {label}
                </Button>
            </div>
        );
        return refreshNode;
    }
    /**
     * Render DataList.Head filter.
     * @since 0.1.2
     * @author liujiali
     * @private
     * @param {Array} filters - DataList.Filter props.
     * @return {ReactNode} DataList.Filter node.
     */
    renderComponentHeadFilters(filters) {
        const {children, float = 'left', multiple} = filters;
        const style = multiple ? {

        } : {
            display: 'flex',
            float: float,
            ...headMarginMap[float],
        };
        return children.length ?
            (<div style={style}>
                <DataListFilterComponent filter={children}
                    onSearch={this.handleFilterChange()}
                    onMultipleSearch={this.handleMultipleFilterChange()}
                    multiple={multiple} />
            </div>) : null;
    }
    /**
     * Render DataList.Head actions.
     * @since 0.1.2
     * @author liujiali
     * @private
     * @param {Array} action - DataList.Action props.
     * @return {ReactNode} DataList.Action node.
     */
    renderComponentHeadActions(action) {
        if (!action) return null;
        // const acts = action.map((i) => i).reverse();
        const {float = 'left'} = action;
        const style = {
            float: float,
            ...headMarginMap[float],
        };
        const {selectedKeys, selectedRows} = this.state;
        let {disabled} = action;
        disabled = exec(disabled, selectedKeys, selectedRows);
        const message = disabled.isDisabled ? disabled.message : null;
        const isDisabled = disabled.isDisabled ? 'disabled' : null;
        return (
            <div style={style}>
                <Tooltip title={message}>
                    <Button type={action.type}
                        disabled={isDisabled}
                        onClick={() => exec(
                            action.onClick, selectedKeys, selectedRows
                        )}
                        style={{ visibility: action.visible ? 'block' : 'hidden' }}
                        >{action.label}</Button>
                </Tooltip>
            </div>
        );
    }
    /**
     * Render DataList.Head actions menus.
     * @since 0.1.2
     * @author liujiali
     * @private
     * @param {Array} menu - DataList.ActionMenu props.
     * @return {ReactNode} DataList.ActionMenu node.
     */
    renderComponentHeadMenus(menu) {
        if (!menu) return null;
        const {label, type, float='left'} = menu;
        const style = {
            float: float,
            ...headMarginMap[float],
        };
        const itemStyle = {
            display: 'inline-block',
            width: '100%',
        };
        const {actions} = menu;
        const menuItem = actions.map((act, index) => {
            let {disabled} = act;
            const {selectedKeys, selectedRows} = this.state;
            disabled = exec(disabled, selectedKeys, selectedRows);
            const message = disabled.isDisabled ? disabled.message : null;
            const item = (
                <span onClick={() => exec(
                    act.onClick, selectedKeys, selectedRows
                )}>
                    {act.label}
                </span>
            );
            const disabledItem = (
                <a disabled='disabled'>
                    {act.label}
                </a>
            );
            return (
                <Menu.Item key={index}>
                    <Tooltip title={message}>
                        <span style={itemStyle}>
                            {disabled.isDisabled ? disabledItem : item}
                        </span>
                    </Tooltip>
                </Menu.Item>
            );
        });
        const actionMenu = (
            <Menu>
                {menuItem}
            </Menu>
        );
        return (
            <Dropdown overlay={actionMenu}>
                <Button style={style} type={type}>
                    {label} <Icon type='down' />
                </Button>
            </Dropdown>
        );
    }
    /**
     * Render filter tags.
     * @since 0.1.4
     * @author liujiali
     * @private
     * @return {ReactNode} Filter tags node.
     */
    renderFilterTags() {
        const {filters} = this.state;
        const style = {marginRight: 8};
        return filters.length ? (
            <Fragment>
                <Icon type='filter' style={style} />
                {filters.map((f, index) => {
                    return (
                        <Tag color='blue'
                            closable
                            onClose={() => this.handleDeleteFilter(index)}
                            key={f.dataKey}>{f.label}: {f.valueLabel}</Tag>
                    );
                })}
            </Fragment>
        ) : null;
    }
    /**
     * Handle close filter tags.
     * @since 0.1.4
     * @author liujiali
     * @private
     * @param {number} index - Filter tags index.
     */
    handleDeleteFilter(index) {
        const {filters} = this.state;
        filters.splice(index, 1);
        this.setState({
            filters: filters,
        }, this.startLoadingData());
    }
    /**
     * Set DataList current filter.
     * @since 0.1.2
     * @author liujiali
     * @private
     * @param  {object} filter - Current filter.
     * @return {function} Filter change handler.
     */
    handleFilterChange() {
        return (filter) => {
            const {filters} = this.state;
            const existFilter = find(
                filters, (i) => i.dataKey === filter.dataKey
            );
            if (existFilter) {
                existFilter.value = filter.value;
                existFilter.valueLabel = filter.valueLabel;
            } else {
                filters.push(filter);
            }
            this.setState({
                filters: filters,
            }, this.startLoadingData());
        };
    }
    /**
     * Set DataList multiple filter.
     * @since 0.2.6
     * @author liujiali
     * @private
     * @param  {object} filter - Filters.
     * @return {function} Filter change handler.
     */
    handleMultipleFilterChange() {
        return (filters) => {
            this.setState({
                filters,
            }, this.startLoadingData());
        };
    }
    /**
     * DataList onChange handle function.
     * @since 0.1.0
     * @author liujiali
     * @private
     * @return {function} DataList onChange handle function.
     */
    handleDataListChange() {
        return (pagination, filters, sorts) => {
            const {pagination: {currentPage, pageSize, total}} = this.state;
            const currentPagination = {
                currentPage: pagination.currentPage || currentPage,
                pageSize: pagination.pageSize || pageSize,
                total: pagination.total || total,
            };
            this.setState({
                pagination: currentPagination,
                filters,
                sorts,
            }, this.startLoadingData());
        };
    }
    /**
     * Handle component onSelect event.
     * @since 0.1.5
     * @author liujiali
     * @private
     * @return {function} onSelect event handler.
     */
    handleDataListSelect() {
        return (keys, rows) => {
            const {rowSelection} = this.props;
            if (isObject(rowSelection)) {
                const {onChange} = rowSelection;
                exec(onChange, keys, rows);
            }
            this.setState({
                selectedKeys: keys,
                selectedRows: rows,
            });
        };
    }
    /**
     * Load DataList data.
     * @since 0.1.5
     * @author liujiali
     * @public
     * @param {boolean} slient - show table loading or not.
     * @return {function} Backward compatible returns.
     */
    loadData(slient) {
        this.startLoadingData()(slient);
        // Backward compatible
        return () => {};
    }
    /**
     * Get selected row data.
     * @since 0.1.5
     * @author liujiali
     * @public
     * @return {Array} Selected row data.
     */
    getSelectedRows() {
        const {selectedRows} = this.state;
        return selectedRows;
    }
    /**
     * Get DataList data.
     * @since 0.2.9
     * @author liujiali
     * @public
     * @return {Array} DataList data.
     */
    getDataListData() {
        const {data} = this.state;
        return data;
    }
    /**
     * Clear all DataList check state.
     * @since 0.3.0
     * @author liujiali
     * @public
     * @return {Object} Previous check state.
     */
    clearAllCheck() {
        const {selectedKeys, selectedRows} = this.state;
        this.setState({
            selectedKeys: [],
            selectedRows: [],
        });
        return {
            selectedKeys,
            selectedRows,
        };
    }
    /**
     * Validate DataList input fields.
     * @since 0.3.2
     * @author liujiali
     * @public
     * @param {string} fields - Validate fields.
     * @param {number} index - Validate data index.
     * @return {Promise} if all fields has no error, the promise resolve, else reject.
     */
    validateFields() {
        return new Promise((resolve, reject) => {
            this.fieldsTrigger.next();
            setTimeout(() => {
                const {data} = this.state;
                let isError = false;
                for (const dat of data) {
                    const {fieldsError={}} = dat;
                    for (const key in fieldsError) {
                        if (fieldsError[key]) {
                            isError = true;
                            break;
                        }
                    }
                    if (isError) {
                        break;
                    }
                }
                isError ? reject() : resolve();
            }, 0);
        });
    }
    /**
     * Manual loading table.
     * @since 0.3.7
     * @author liujiali
     * @public
     * @param {boolean} loading - Validate fields.
     */
    startLoading(loading = true) {
        this.setState({
            loading: loading,
        });
    }
    /**
     * Render method.
     * @since 0.1.0
     * @author liujiali
     * @private
     * @return  {*} No return value.
     */
    render() {
        let {type, children} = this.props;
        type = isBlank(type) ? 'TABLE' : type.toUpperCase();
        let title = this.getDefineFromProps(children, DataListTitle);
        title = title.length ? title[0] : {};
        let head = this.getDefineFromProps(children, DataListHead);
        head = head.length ? head[0] : {};
        return (
            <Fragment>
                {this.renderComponentTitle(title)}
                {this.renderComponentHead(head)}
                {this.renderComponet(type === 'TABLE', DataListTable)}
            </Fragment>
        );
    }
}

export default DataList;

/**
 * beforeLoadData method description.
 * @callback beforeLoadData
 * @since 0.1.0
 * @author liujiali
 * @param {*} data - props `data` in DataList.
 * @param {object} pagination - Pagination information.
 * @param {number} pagination.currentPage - Current page number.<br/>Start from 1.
 * @param {number} pagination.pageSize - Number of data items per page.
 * @param {number} pagination.total - Total item count.
 * @param {object[]} filters - Current table filter items.
 * @param {string} filters.dataKey - Filter column data key.
 * @param {string} filters.label - Filter column label.
 * @param {string} filters.value - Filter column value.
 * @param {string} filters.valueLabel - Filter column value label.
 * @param {enum} filters.type - Filter type.<br/>[`text`, `range`, `enum`, `value`].
 * @param {object[]} sorts - Current table sort items.
 * @param {string} sorts.key - Sort column key.
 * @param {enum} sorts.value - Sort column order.<br/>[`ascend`, `descend`].
 * @return {Array} - Return [data, pagination, filters, sorts, ...args] as parameter for handleRequestData.
 */
/**
 * afterLoadData method description.
 * @callback afterLoadData
 * @since 0.1.0
 * @author liujiali
 * @param  {*} data - props `data` in DataList.
 * @param  {object} pagination - Pagination information.
 * @param  {number} pagination.currentPage - Current page number.<br/>Start from 1.
 * @param  {number} pagination.pageSize - Number of data items per page.
 * @param  {object[]} filters - Current table filter items.
 * @param  {string} filters.dataKey - Filter column data key.
 * @param {string} filters.label - Filter column label.
 * @param  {string} filters.value - Filter column value.
 * @param {string} filters.valueLabel - Filter column value label.
 * @param  {enum} filters.type - Filter type.<br/>[`text`, `range`, `enum`, `value`].
 * @param  {object[]} sorts - Current table sort items.
 * @param  {string} sorts.key - Sort column key.
 * @param  {enum} sorts.value - Sort column order.<br/>[`ascend`, `descend`].
 * @return {Array} - Return [data, pagination, filters, sorts, ...args] as parameter for handleRequestData.
 */
