import React from 'react';
import {isBlank, exec} from 'rc-js-utils';
import {Badge} from 'antd';
import {isString} from 'lodash';

/**
 * Detail.Column component.
 * @memberof Detail
 * @name Detail.Column
 * @description Define the Detail column.
 * @since 0.2.4
 * @author liujiali
 * @property {string} [label=null] - Column label.
 * @property {string} dataKey - Column data key.
 * @property {render} [render=function(value) { return value ? value : '-' }] - Column render method.
 * @property {boolean} [visible=true] - Whether column is visible.
 * @property {string} [type] - Column type.Effective when type=`list`.
 * @property {string|object} [convert=null] - Several predefined render method.[`date`, `status`, `number`, `string`].
 *                                            <br/>If the `render` property is defined, this props will not be used.
 * @property {enum} convert.type - Convert type.[`date`, `status`, `number`, `string`].
 * @property {string} [convert.format=`YYYY-MM-DD HH:mm:ss`] - Effective when type=`date`.<br/>Date format.
 * @property {number} [convert.timezone=local timezone] - Effective when type=`date`.<br/>Date convert target timezone.
 * @property {object} [convert.statusMap] - Effective when type=`status`.<br/>Status map.
 * @property {string} convert.statusMap.key - Status map key.
 * @property {string} convert.statusMap.value - Status map value.<br/>[`success`, `warning`, `error`, `processing`, `default`].
 * @property {function} [convert.render] - Status label render.
 * @property {string} [help] - Column help message.
 * @property {boolean} [colon=true] - Wheather show colon or not.
 * @example
 * <Detail>
 *     <Detail.ColumnGroup>
 *         <Detail.Column label='Name' dataKey='name' render: val => val />
 *         ......
 *     </Detail.ColumnGroup>
 *     ......
 * </Detail>
 */
class DetailColumn {
    /**
     * Create and initialize component.
     * @param {*} props - Component props.
     */
    constructor(props) {
        const defaultAttrs = {
            label: null,
            dataKey: null,
            render: this.getDefaultRender(props),
            visible: true,
            convert: null,
            help: null,
        };
        this.attrs = Object.assign(defaultAttrs, props);
    }
    /**
     * Return default render method for column.
     * @since 0.1.1
     * @author liujiali
     * @private
     * @param {*} props - Component props.
     * @return {render} Column render method.
     */
    getDefaultRender(props) {
        const defaultRender = (value) => isBlank(value) ? '-' : value;
        if (!props.convert) return defaultRender;
        let convert = {};
        isString(props.convert) ?
            convert.type = props.convert : convert = props.convert;
        return this.getConvertRender(convert);
    }
    /**
     * Return render method by convert type.
     * @since 0.1.1
     * @author liujiali
     * @private
     * @param {object} convert - Column convert property.
     * @return {render} Column render method.
     */
    getConvertRender(convert) {
        const type = convert.type;
        let render;
        switch (type) {
        // `string` type.
        case 'string':
            render = (value) => isBlank(value) ? '-' : formatString(value);
            break;
            // `number` type.
        case 'number':
            render = (value) => isBlank(value) ? '-' : formatNumber(value);
            break;
            // `date` type.
        case 'date':
            const {format, timezone} = convert;
            const opts = {};
            if (format) opts.format = format;
            if (!isBlank(timezone)) opts.timezone = timezone;
            render = (value) => isBlank(value) ?
                '-' : convertToLocalDate(value, opts);
            break;
            // `string` type.
        case 'status':
            let {statusMap} = convert;
            if (!statusMap) {
                statusMap = {
                    success: 'success',
                    error: 'error',
                    default: 'default',
                    processing: 'processing',
                    warning: 'warning',
                };
            }
            const statusRender = convert.render || ((val) => formatString(val));
            render = (value) => isBlank(value) ?
                '-' : <Badge status={statusMap[value]}
                    text={exec(statusRender, value)} />;
            break;
        default:
            render = (value) => isBlank(value) ? '-' : value;
        };
        return render;
    }
}

export default DetailColumn;
