import React, {Component, Fragment} from 'react';
import {Row, Col} from 'antd';
import {exec} from 'rc-js-utils';
import DetailTitle from './DetailTitle';
import DetailColumnGroup from './DetailColumnGroup';
import DetailColumn from './DetailColumn';
import {isBoolean} from 'lodash';

const DetailTitleStyle = {
    fontSize: 20,
    lineHeight: '28px',
};

const DetailLabelStyle = {
    minHeight: 36,
    lineHeight: '36px',
    paddingLeft: 12,
    paddingRight: 12,
};

const DetailBorder = {
    borderBottom: '1px solid #e8e8e8',
};

/**
 * Detail component.
 * @namespace Detail
 * @description Define the Detail.
 * @since 0.2.4
 * @author liujiali
 * @property {object} data - Detail data.
 * @example
 * <Detail data={{name: 'Name'}}>
 *     <Detail.Title title='Detail' />
 *     <Detail.ColumnGroup>
 *         <Detail.Column label='Name' dataKey='name' />
 *     </Detail.ColumnGroup>
 *     ......
 * </Detail>
 */
class DetailList extends Component {
    /**
     * Create and initialize component.
     * @param {*} props - Component props.
     */
    constructor(props) {
        super(props);
    }

    static Title = DetailTitle;
    static ColumnGroup = DetailColumnGroup;
    static Column = DetailColumn;

    /**
     * Get the definition of column, action and etc. by parameters.
     * @since 0.1.0
     * @author liujiali
     * @private
     * @param  {ReactNode|ReactNode[]} children - Component children nodes.
     * @param  {class} targetClass - Component class.
     * @return {Array} Component definition list.
     */
    getDefineFromProps(children, targetClass) {
        const attrs = [];
        React.Children.map(children, (child) => {
            const ChildComponent = child.type;
            const childIns = new ChildComponent(child.props);
            if (childIns instanceof targetClass) {
                attrs.push(childIns.attrs);
            }
        });
        return attrs;
    };
    /**
     * Render method.
     * @since 0.2.4
     * @author liujiali
     * @private
     * @return  {*} No return value.
     */
    render() {
        const {children, data} = this.props;
        let title = this.getDefineFromProps(children, DetailTitle);
        const columnGroup = this.getDefineFromProps(
            children, DetailColumnGroup
        );
        title = title.length ? title[0] : {};
        const colSpan = 24 / columnGroup.length;
        const {bordered} = this.props;
        return (
            <Fragment>
                {title.title || title.children ? (
                    <Row style={{marginBottom: 16}}>
                        <Col style={DetailTitleStyle}>{title.title}</Col>
                        <Col>{title.children}</Col>
                    </Row>
                ) : null}
                <Row>
                    {columnGroup.map((colGrp, index) => {
                        const {labelWidth} = colGrp;
                        return (
                            <Col span={colSpan} key={index}>
                                {colGrp.columns.map((col, index) => {
                                    const labelStyle = {
                                        ...DetailLabelStyle,
                                        width: labelWidth,
                                    };
                                    const dataStyle = {
                                        ...DetailLabelStyle,
                                        width: `calc(100% - ${labelWidth}px)`,
                                    };
                                    const value = data[col.dataKey];
                                    let colon = true;
                                    if (isBoolean(col.colon) && !col.colon) {
                                        colon = false;
                                    }
                                    return (
                                        <Row key={index} style={bordered ? DetailBorder : {}}>
                                            <Col style={labelStyle} span={1}>
                                                {col.label}{colon ? ':' : ''}
                                            </Col>
                                            <Col style={dataStyle} span={1} >
                                                {exec(col.render, value)}
                                            </Col>
                                        </Row>
                                    );
                                })}
                            </Col>
                        );
                    })}
                </Row>
            </Fragment>
        );
    }
}

export default DetailList;
