import React, {Fragment, Component} from 'react';
import {Modal, Button} from 'antd';
import {getDefaultSettings} from '../index';

const LOCALE = {
    zh: {
        cancelButton: '取消',
    },
    en: {
        cancelButton: 'Cancel',
    },
};

/**
 * Help you to use antd Modal more conveniently.
 * @since 0.2.0
 * @author liujiali
 * @property {object} modal - `modal` props for injected component.
 * @property {showModal} modal.showModal - Show Modal function.
 * @property {function} modal.closeModal - Close Modal function.
 * @property {object} modal - `modal` props for Modal component.
 * @property {string} [modal.title] - Modal title.
 * @property {number} [modal.width] - Modal width.
 * @property {Array} [modal.footer] - Modal footer.The modal will always has `Cancel` button.
 * @example
 * // first inject to the component.
 * class Demo extends Component {
 *     ......
 * }
 * export default ModalHelper()(Demo);
 *
 * // then use `modal` in props to open the modal.
 * class Demo extends Component {
 *     handleAddClick() {
 *         const {modal} = this.props;
 *         modal.showModal({
 *             title: 'Add',
 *             width: 700,
 *             component: <AddModal />,
 *         });
 *     }
 * }
 *
 * // you can use `modal` props in modal component to set the Modal footer or control the modal close.
 * class AddModal extends Component {
 *     componentDidMount() {
 *         const {modal} = this.props;
 *         modal.footer = [
 *             <Button onClick={this.handleDoneClick}>Done</Button>
 *         ];
 *     }
 *     handleDoneClick = () => {
 *         const {modal} = this.props;
 *         modal.closeModal();
 *     }
 * }
 * @return {function} - Inject funtion.
 */
const ModalHelper = () => {
    return function(WrappedComponent) {
        /**
         * ModalHelper component.
         * @namespace ModalHelper
         * @description This component is used to use Modal but not write <Modal>
        */
        class ModalHelper extends Component {
            /**
             * Create and initialize component.
             * @param {*} props - Component props.
             */
            constructor(props) {
                super(props);
                this.state = {
                    title: '',
                    showModal: false,
                    width: 500,
                    component: null,
                    footer: [],
                    // afterClose: null,

                    isDrag: false,
                    mousePageX: undefined,
                    mousePageY: undefined,
                };
            }
            /**
             * Show modal.
             * @since 0.2.0
             * @author liujiali
             * @private
             * @return {function} Handle Modal open.
             */
            showModal() {
                return (option) => {
                    this.setState({
                        showModal: true,
                        title: option.title,
                        width: option.width ? option.width : 500,
                        component: option.component ? option.component : null,
                        // footer: footer,
                        // afterClose: option.afterClose ? option.afterClose : null
                    });
                };
            }
            /**
             * Hide modal.
             * @since 0.2.0
             * @author liujiali
             * @private
             * @return {function} Handle Modal close.
             */
            closeModal() {
                return () => {
                    this.setState({
                        showModal: false,
                    });
                };
            }
            /**
             * Re-render the modal.
             * @since 0.2.3
             * @author liujiali
             * @private
             * @return {function} Handle Modal re-render.
             */
            reloadModal() {
                return () => {
                    this.forceUpdate();
                };
            }
            /**
             * Get Modal dragable header.
             * @since 0.2.0
             * @author liujiali
             * @private
             * @return {function} Antd Modal title function.
             */
            getModalHeader() {
                const style = {
                    padding: '12px 0 12px 24px',
                    margin: '-12px 0 -12px -24px',
                };
                return (
                    <div onMouseDown={this.handleDragMouseDown()}
                        onMouseMove={this.handleDragMouseMove()}
                        onMouseUp={this.handleDragMouseUp()}
                        onMouseLeave={this.handleDragMouseUp()}
                        style={style}
                    >{this.state.title}</div>
                );
            }
            /**
             * Handle modal drag start.
             * @since 0.2.0
             * @author liujiali
             * @private
             * @return {function} Modal drag handler.
             */
            handleDragMouseDown() {
                return (e) => {
                    // TODO: use class to locale the modal
                    const modalElement = e.target.parentElement.parentElement
                        .parentElement;
                    this.setState({
                        ...this.state,
                        isDrag: true,
                        mousePageX: e.pageX - modalElement.offsetLeft,
                        mousePageY: e.pageY - modalElement.offsetTop,
                    });
                };
            }
            /**
             * Handle modal drag over.
             * @since 0.2.0
             * @author liujiali
             * @private
             * @return {function} Modal drag handler.
             */
            handleDragMouseUp() {
                return () => {
                    this.setState({
                        ...this.state,
                        isDrag: false,
                    });
                };
            }
            /**
             * Handle modal drag move.
             * @since 0.2.0
             * @author liujiali
             * @private
             * @return {function} Modal drag handler.
             */
            handleDragMouseMove() {
                return (e) => {
                    if (!this.state.isDrag) return;
                    const modalNode = e.target.parentElement.parentElement
                        .parentElement;
                    const {style} = modalNode;
                    style.top = `${e.pageY - this.state.mousePageY}px`;
                    style.left = `${e.pageX - this.state.mousePageX}px`;
                };
            }
            /**
             * Get Modal child component.
             * @since 0.2.0
             * @author liujiali
             * @private
             * @param {function} component - Modal child component
             * @return {ReactNode} Child component node.
             */
            getModalComponent(component) {
                if (!component || !component.type) return null;
                const Component = component.type;
                const {props} = component;
                const modal = {
                    footer: [],
                    showModal: this.showModal(),
                    closeModal: this.closeModal(),
                    reload: this.reloadModal(),
                };
                const modalProxy = new Proxy(modal, {
                    set: (target, propKey, value, receiver) => {
                        this.setState({
                            [propKey]: value,
                        });
                        return true;
                    },
                });
                return <Component {...props} modal={modalProxy} />;
            }
            /**
             * Render method.
             * @since 0.2.0
             * @author liujiali
             * @private
             * @return  {*} No return value.
             */
            render() {
                const nextProps = {
                    ...this.props,
                    modal: {
                        showModal: this.showModal(),
                        closeModal: this.closeModal(),
                        reload: this.reloadModal(),
                    },
                };
                const component = this.getModalComponent(this.state.component);
                const {footer} = this.state;
                const footers = footer.map((i) => i);
                const settings = getDefaultSettings();
                const {locale} = settings;
                footers.push(
                    <Button onClick={this.closeModal()}
                        key='cancel'>{LOCALE[locale].cancelButton}</Button>
                );
                return (
                    <Fragment>
                        <WrappedComponent {...nextProps} />
                        <Modal title={this.getModalHeader()}
                            visible={this.state.showModal}
                            footer={footers}
                            destroyOnClose
                            maskClosable={false}
                            width={this.state.width}
                            onCancel={() => this.closeModal()()}>
                            {component}
                        </Modal>
                    </Fragment>
                );
            }
        }
        return ModalHelper;
    };
};

export default ModalHelper;

/**
 * showModal method description.
 * @callback showModal
 * @since 0.2.0
 * @author liujiali
 * @param {object} option - Modal option.
 * @param {string|ReactNode} [option.title] - Modal title.
 * @param {number} [option.width=500] - Modal width.
 * @param {ReactNode} option.component - Modal component.
 */
